function varargout = warpgui(varargin)
%   function varargout = warpgui(varargin)
%
%   Purpose:
%   Perform warping on a vector of data. 
%   Data is read from MATLAB memory.
%
%   Description:
%   The settings of the program are kept in the struct "GlobSet".
%   The meaning of the fields:
%
%
%       GlobSet.GlobalConstraints    - scalar.
%                                               0 = Use global constraints.
%                                               1 = Don't use globa constraints.
%                                               Default: 0
%                       
%       GlobSet.Normalize            - scalar.
%                                               0 = No normalization.
%                                               1 = Normalization.
%                                               Default: 0
%
%       GlobSet.DirPenalties         - scalar.
%                                               0 = Use direction penalty.
%                                               1 = Don't use direction penalty.
%                                               Default: 0
%
%       GlobSet.DirPenaltiesVal      - scalar.
%                                               Value for direction penalty
%                                               Default: 12
%
%       GlobSet.RawData              - matrix with raw data.
%       GlobSet.WarpedData           - matrix with warped data.
%       GlobSet.WarpedPath           - vector?
%       GlobSet.AxesLimits           - (4 x 1) vector
%                                      axes limits
%
%   It is assumed that data is stored columnwise.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
%
if ( nargin == 0 | (nargin == 1 & isstruct(varargin{1})) )
    % No argument or GlobSet as an input argument

    if ( getMLver < 6.0 )
        errordlg('MATLAB version 6.0 or higher is needed to run this program')
        return
    end
    
    fig = openfig(mfilename,'new');
    
    
    % Generate a structure of handles to pass to callbacks, and store it. 
    handles              = guihandles(fig);
    
    switch ( nargin  )
    case 0

        %   Set the DEAFAULT values for the options
        GlobSet.SampleSelection                 = 1;
        GlobSet.ExampleDataRead                 = 0;
        GlobSet.Version                         = '2.1';  % Version number software
        GlobSet.GlobConstraints                 = 1;
        GlobSet.Normalize                       = 0;
        GlobSet.DirPenalties                    = 0;
        GlobSet.DirPenaltiesVal                 = 12;
        GlobSet.PlotCounter                     = 1;
        GlobSet.RawData                         = [];
        GlobSet.WarpedData                      = [];
        GlobSet.WarpedPath                      = [];
        GlobSet.VarNameRef                      = [];
        GlobSet.VarNameSample                   = [];
        GlobSet.Constraints                     = [GlobSet.GlobConstraints,...
                                                   GlobSet.Normalize,...
                                                   GlobSet.DirPenalties,...
                                                   GlobSet.DirPenaltiesVal];
        GlobSet.PrevFigureColor                 = get(0,'DefaultFigureColor');
        GlobSet.PrevUicontrolBackgroundColor    = get(0,'defaultUicontrolBackgroundColor');
        GlobSet.BackgroundColor                 = [0.984,0.473,0.012];
        GlobSet.ForegroundColor                 = [0,0,0];
        GlobSet.HelpDir                         = [pwd,'\help'];
        
        handles.GlobSet                         = GlobSet;
        
    case 1
    
        handles.GlobSet = varargin{1};
        
    end 
    guidata(handles.warp_GUI, handles);
    Initialize(handles);
    if nargout > 0
        varargout{1} = fig;
    end    


elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
        
    catch
        disp(lasterr);
    end

end

% --------------------------------------------------------------------
function Initialize(handles)

    GlobSet = handles.GlobSet;
    
    if ( ~GlobSet.ExampleDataRead )
        %   Read the example data, if not done already.
        evalin('base','load examples.mat');
        GlobSet.ExampleDataRead = 1;
        
        %   Read Logo's.
        GlobSet.TipbLogo   = imread('tipblogo.bmp');
        GlobSet.UvALogo    = imread('uvalogo.bmp');
    end        
    
    %   Show the logo's.
    axes(handles.UvALogo);
    image(GlobSet.UvALogo);    
    axis off
           
    axes(handles.TipbLogo);
    image(GlobSet.TipbLogo);
    axis off
    
    %   Set Version Text.
    set(handles.TEXTversion,'String',['version ',GlobSet.Version]);
        
    %   Default plot options.
    set(0,'DefaultAxesLinewidth',2);
    set(0,'DefaultLineLinewidth',2);
    set(0,'DefaultFigureColor'              ,GlobSet.BackgroundColor);
    set(0,'DefaultUicontrolBackgroundColor' ,GlobSet.BackgroundColor);
    set(0,'DefaultUicontrolForegroundColor' ,GlobSet.ForegroundColor);
    
    %   Initialize the screen settings.
    set(handles.GlobConstraints ,'Value',GlobSet.GlobConstraints);
    set(handles.Normalize       ,'Value',GlobSet.Normalize);
    set(handles.DirPenalties    ,'Value',GlobSet.DirPenalties);
    
    
    if ( GlobSet.DirPenalties ) 
    
        %   Set Direction penalties value and make it active.
        set(handles.DirPenaltiesVal,'String',num2str(GlobSet.DirPenaltiesVal));
        set(handles.DirPenaltiesVal,'Visible','on');
        
    else        
        
        %   Set Direction penalties value and make it in-active.
        set(handles.DirPenaltiesVal,'String',num2str(GlobSet.DirPenaltiesVal));
        set(handles.DirPenaltiesVal,'Visible','off');
        
    end
    
    DataKnown = ( ~isempty(GlobSet.VarNameSample) &  ~isempty(GlobSet.VarNameRef));
       
    if ( DataKnown )
    
        %   2.  Read the data and check size of data.
        GlobSet.RawData.Sample  = GetVarFromBase(GlobSet.VarNameSample);
        GlobSet.RawData.Ref     = GetVarFromBase(GlobSet.VarNameRef);
        nvar_Sample = size(GlobSet.RawData.Sample,2);        
        nvar_Ref    = size(GlobSet.RawData.Ref,2);
        
        if ( nvar_Sample ~= nvar_Ref )
        
            errordlg('Same number of variables required')
            GlobSet.VarNameSample   = [];
            GlobSet.VarNameRef      = [];
            
        else
        
            %   Get a sensible Directional Penalty.
            [temp1,temp2,temp3,GlobSet.DirPenaltiesVal] = InitGrid(GlobSet.RawData.Sample,GlobSet.RawData.Sample,GlobSet.GlobConstraints);

            %   3.  Plot data in RawDataPlot figure.
            axes(handles.RawDataPlot)
            if ( nvar_Sample == 1 ) 
                plot(GlobSet.RawData.Sample,'r-')
            else
                hsam = plot(GlobSet.RawData.Sample,'-');            
            end
            hold on
            if ( nvar_Ref == 1)
                plot(GlobSet.RawData.Ref,'b-')
                hold off
                legend('Sample','Ref',0)            
            else                
                href = plot(GlobSet.RawData.Ref,':');
                hold off
                legend([hsam(1),href(1)],'Sample','Ref',0)            
            end                
            title('Raw Data')
            xlabel('Independent variable');
            GlobSet.AxesLimits = paxis(axis);
            axis(GlobSet.AxesLimits)

            %   4.  Empty plot with warped data.
            delete(get(handles.WarpedDataPlot,'children'));
            axes(handles.WarpedDataPlot);
            axis(GlobSet.AxesLimits);

            %   5.  Empty plot with warping path and make it invisible.
            delete(get(handles.WarpedPathPlot,'children'));
            set(handles.WarpedPathPlot,'visible','off')
        end            
        
    end
    
    %   Handle directional penalties option.
    if ( GlobSet.DirPenalties ) 
        
        %   Set Direction penalties value and make it active
        set(handles.DirPenaltiesVal,'String',num2str(GlobSet.DirPenaltiesVal));
        set(handles.DirPenaltiesVal,'Visible','on');

    else        

        %   Set Direction penalties value and make it in-active
        set(handles.DirPenaltiesVal,'String',num2str(GlobSet.DirPenaltiesVal));
        set(handles.DirPenaltiesVal,'Visible','off');

    end

    %   Store changes in figure memory space.
    handles.GlobSet = GlobSet;
    guidata(handles.warp_GUI,handles);

% --------------------------------------------------------------------
function varargout = GlobConstraints_Callback(handles)
%   Handle  Global constraints check box
    handles.GlobSet.GlobConstraints = get(handles.GlobConstraints,'Value');    
    handles.GlobSet.Constraints(1)  = handles.GlobSet.GlobConstraints;
    guidata(handles.warp_GUI,handles);
    
% --------------------------------------------------------------------
function varargout = Normalize_Callback(handles)
%   Handle normalization check box
    handles.GlobSet.Normalize = get(handles.Normalize,'Value');
    handles.GlobSet.Constraints(2)  = handles.GlobSet.Normalize;
    guidata(handles.warp_GUI,handles);

% --------------------------------------------------------------------
function varargout = DirPenalties_Callback(handles)
%   Handle Direction penalties check box.
    handles.GlobSet.DirPenalties = get(handles.DirPenalties,'Value');
    if ( handles.GlobSet.DirPenalties ) 
        %   Make Direction penalties value active
        set(handles.DirPenaltiesVal,'Visible','on');
    else        
        %   Make Direction penalties value in-active
        set(handles.DirPenaltiesVal,'Visible','off');
    end
    handles.GlobSet.Constraints(3)  = handles.GlobSet.DirPenalties;
    guidata(handles.warp_GUI,handles);
    
% --------------------------------------------------------------------
function varargout = DirPenaltiesVal_Callback(handles)
%   Handle edit field    
    handles.GlobSet.DirPenaltiesVal = str2num(get(handles.DirPenaltiesVal,'string'));
    handles.GlobSet.Constraints(4)  = handles.GlobSet.DirPenaltiesVal;
    guidata(handles.warp_GUI,handles);

% --------------------------------------------------------------------
function varargout = PlottedSignal_Callback(handles)

    GlobSet = handles.GlobSet;
    
    %   Read signal number to plot.
    ivar = str2num(get(handles.PlottedSignal,'String'));
    [npts,nvar] = size(GlobSet.Results.WarpedSample);
    if ( ivar > nvar )
        ivar = nvar;
    end
    if ( ivar < 1 )
        ivar = 1;
    end
    GlobSet.PlotCounter = ivar;
    set(handles.PlottedSignal,'string',num2str(ivar));
    
    handles.GlobSet = GlobSet;
    guidata(handles.warp_GUI,handles);
        
% --------------------------------------------------------------------
function varargout = PlotSingle_Callback(handles)
%   Plot a single warped curve.
    GlobSet = handles.GlobSet;
    
    %   Read signal number to plot.
    ivar = GlobSet.PlotCounter;
    set(handles.PlottedSignal,'string',num2str(ivar));
    
    %   Do plot it
    axes(handles.WarpedDataPlot);
    cla
    plot(GlobSet.Results.WarpedSample(:,ivar),'r-')
    hold on
    plot(GlobSet.Results.WarpedRef(:,ivar),'b-')    
    title('Warped Data');
    xlabel('Joint axis [-]');
    legend('Sample','Ref',0)            

    axes(handles.RawDataPlot)
    cla
    plot(GlobSet.RawData.Sample(:,ivar),'r-')
    hold on
    plot(GlobSet.RawData.Ref(:,ivar),'b-')
    title('Raw Data')
    xlabel('Independent variable');
    legend('Sample','Ref',0)            
           
    %   and save.
    [npts,nvar] = size(GlobSet.Results.WarpedSample);
    ivar = ivar+1;
    if ( ivar > nvar )
        ivar = 1;
    end
    GlobSet.PlotCounter = ivar;
    
    handles.GlobSet = GlobSet;
    guidata(handles.warp_GUI,handles);

        
% --------------------------------------------------------------------
function UpdateWarp(handles)
    GlobSet = handles.GlobSet;


    WarpMode = 'Unknown';
    if ( get(handles.DTWChoice,'Value') )    
        WarpMode = 'DTW';
    end
    if ( get(handles.PTWChoice,'Value') )    
        WarpMode = 'PTW';
    end
    
    %   0.  plot raw data
    axes(handles.RawDataPlot)
    hold off
    if ( size(GlobSet.RawData.Sample,2) == 1 ) 
        plot(GlobSet.RawData.Sample,'r-')
    else
        hsam = plot(GlobSet.RawData.Sample,'-');            
    end
    hold on
    if ( size(GlobSet.RawData.Ref,2) == 1 ) 
        plot(GlobSet.RawData.Ref,'b-')
        hold off
        legend('Sample','Ref',0)            
    else                
        href = plot(GlobSet.RawData.Ref,':');
        hold off
        legend([hsam(1),href(1)],'Sample','Ref',0)            
    end                
    title('Raw Data')
    xlabel('Independent variable');
    GlobSet.AxesLimits = paxis(axis);
    axis(GlobSet.AxesLimits)

    %   1.  Warp the RAW data.
    GlobSet.Results = WarpData(GlobSet.RawData,handles.GlobSet.Constraints,WarpMode);
    
    %   2.  Plot results in warped data plot.
    [npts,nvar_Bsam] = size(GlobSet.Results.WarpedSample);
    nvar_Bref = size(GlobSet.Results.WarpedRef,2);
    if ( nvar_Bref > 1 & strcmp(WarpMode,'DTW') )
        set(handles.PlotSingle,'enable','on');
        set(handles.PlottedSignal,'enable','on');
    else
        set(handles.PlotSingle,'enable','off');
        set(handles.PlottedSignal,'enable','off');
    end        
    
    axes(handles.WarpedDataPlot);
    hold off
    if ( nvar_Bsam == 1)
        plot(GlobSet.Results.WarpedSample,'r-')
    else
        plot(GlobSet.Results.WarpedSample,'-')
    end        
    hold on
    if ( nvar_Bref == 1)
        plot(GlobSet.Results.WarpedRef,'b-')
    else
        plot(GlobSet.Results.WarpedRef,':')
    end
    title('Warped Data');
    xlabel('Joint axis [-]');
    legend('Sample','Ref',0)      
    hold off
    axis([0 length(GlobSet.Results.WarpedPath) GlobSet.AxesLimits(3) GlobSet.AxesLimits(4)]);    
       
    %   3.  Generate a plot of the warping path
    axes(handles.WarpedPathPlot)
    hold off
    set(handles.WarpedPathPlot,'visible','on')
    switch( upper(WarpMode ) )
    case {'DTW'}
    
        Xpath = GlobSet.Results.WarpedPath(:,1);
        Ypath = GlobSet.Results.WarpedPath(:,2);
        plot(Xpath,Ypath,'k-','linewidth',2);
        xmax = max(Xpath);
        ymax = max(Ypath);
        ylabel('Reference Coordinate')
        xlabel('Sample Coordinate')
        title('Warping function');
        axis([0,xmax,0,ymax]);
    
        
    case {'PTW'}        
    
        plot(GlobSet.Results.WarpedPath,'k-','linewidth',2)
        xmax = length(GlobSet.Results.WarpedPath);
        ymax = xmax;
        ylabel('Reference Coordinate')
        xlabel('Sample Coordinate')
        title('Warping function');
        axis([0,xmax,0,ymax]);
            
    end
    hold on
    grid on
    
    if ( handles.GlobSet.GlobConstraints )
    
        %   Draw limits when normalization option is on.            
        Xsample     = [ 0 : length(GlobSet.RawData.Ref)];
        Ysample     = Xsample + GlobSet.Results.GlobalConstraints(2);
        
        Xreference  = [ 0 : length(GlobSet.RawData.Ref)];
        Yreference  = Xreference - GlobSet.Results.GlobalConstraints(1);
        
        plot(Xsample,Ysample,'r','linewidth',2)
        plot(Xreference,Yreference,'r','linewidth',2)
        xmax = max([xmax,Xreference,Xsample]);
        ymax = max([ymax,Ysample,Yreference]);
    
    end
    ylabel('Reference Coordinate')
    xlabel('Sample Coordinate')
    title('Warping function');
    axis([0,xmax,0,ymax]);
    hold off
       
    %   4.  Export GlobSet struct to main MATLAB environment.
    %       and to figure itself.
    assignin('base','Results',GlobSet.Results);
    
    handles.GlobSet = GlobSet;
    guidata(handles.warp_GUI,handles);

% --------------------------------------------------------------------
function varargout = BUTwarp_Callback(handles)
    GlobSet     = handles.GlobSet;
    DataKnown   = ( ~isempty(GlobSet.VarNameSample) &  ~isempty(GlobSet.VarNameRef));
    if ( DataKnown )
        y=wavread('maxwarp');
        sound(y,85000)
        UpdateWarp(handles)
    else
        errordlg('No data yet. Use "Load Data" button first');
    end
    
% --------------------------------------------------------------------
function BUTLoadData_Callback(handles)
    % Get the global settings struct
    GlobSet                = handles.GlobSet;
    
    %   Close this GUI
    close(handles.warp_GUI);
    
    %   Open selection of variables.
    GetMemVar(GlobSet);
    
% --------------------------------------------------------------------        
function BUTprint_Callback(handles)

    % Get the global settings struct
    GlobSet                = handles.GlobSet;
    
    DataKnown   = ( ~isempty(GlobSet.VarNameSample) &  ~isempty(GlobSet.VarNameRef));
    
    if ( DataKnown )

        %   Prepare for printing info about settings and data that are loaded
    
        %   Find all UIcontrol objects and make them invisible.
        hControls = findobj(handles.warp_GUI,'type','UIcontrol');
        set(hControls,'visible','off')
        
        %   Change the frame UIcontrol into a text UIcontrol
        hframe                  = findobj(hControls,'style','frame');
        BackupBackgroundcolor   = get(hframe,'backgroundcolor');
        BackupForegroundcolor   = get(hframe,'foregroundcolor');
        set(hframe,'style','text',...
                   'visible','on',...
                   'backgroundcolor','w',...
                   'foregroundcolor','k');
                
        %   Generate the text to be printed.
        GlobalConstraintsSet = 'OFF';
        if (  GlobSet.GlobConstraints )
            GlobalConstraintsSet = 'ON';
        end            
        
        NormalizeSet = 'OFF';
        if (  GlobSet.Normalize )
            NormalizeSet = 'ON';
        end            
        
        DirPenaltiesSet = 'OFF';
        if (  GlobSet.DirPenalties )
            DirPenaltiesSet = 'ON';
        end            

        TempText         = char('Settings:',...
                                ['  Global Constraints  ',GlobalConstraintsSet],...
                                ['  Normalize           ',NormalizeSet],...
                                ['  Direction Penalties ',DirPenaltiesSet],...
                                sprintf('    Penalty value:  %8.3f',GlobSet.DirPenaltiesVal),...
                                ' ',...
                                sprintf('Sample:     %s',GlobSet.VarNameSample),...
                                sprintf('Reference:  %s',GlobSet.VarNameRef));
                                
        % Print text in UIcontrol and update screen.
        set(hframe,'string',TempText);
        
        % Change Title colors
        htitle      = findobj(hControls,'tag','TEXTtitle');
        BBackcolor  = get(htitle,'backgroundcolor');
        BForecolor  = get(htitle,'foregroundcolor');
        set(htitle,'backgroundcolor','w','foregroundcolor','k','visible','on');
        hversion = findobj(hControls,'tag','TEXTversion');
        set(hversion,'backgroundcolor','w','foregroundcolor','k','visible','on');
        drawnow
        
        %   Issue print command
        print(handles.warp_GUI)
        
        %   Reset UIcontrol objects, make sure that frame is drawn last. 
        set(hframe,'style','frame',...
                   'backgroundcolor',BackupBackgroundcolor,...
                   'foregroundcolor',BackupForegroundcolor);
        set(htitle  ,'backgroundcolor',BBackcolor,'foregroundcolor',BForecolor);                   
        set(hversion,'backgroundcolor',BBackcolor,'foregroundcolor',BForecolor);

        set(hControls,'visible','on')
        hChild = get(handles.warp_GUI,'children');
        set(handles.warp_GUI,'children',hChild);
        drawnow
        
    else
    
        errordlg('Nothing to print yet. Use "Load Data" button first');
        
    end
            
% --------------------------------------------------------------------
function PTWChoice_Callback(handles)

WarpChoice  = get(handles.PTWChoice,'Value');
if (WarpChoice)
    set(handles.DTWChoice,'Value',0);
    set(handles.GlobConstraints,'enable','off');
    set(handles.Normalize,'enable','off');
    set(handles.DirPenalties,'enable','off');
    set(handles.DirPenaltiesVal,'enable','off');
else
    set(handles.DTWChoice,'Value',1);
end
guidata(handles.warp_GUI,handles);
    
% --------------------------------------------------------------------
function DTWChoice_Callback(handles)

WarpChoice  = get(handles.DTWChoice,'Value');
if (WarpChoice)
    set(handles.PTWChoice,'Value',0);
    set(handles.GlobConstraints,'enable','on');
    set(handles.Normalize,'enable','on');
    set(handles.DirPenalties,'enable','on');
    set(handles.DirPenaltiesVal,'enable','on');
else
    set(handles.PTWChoice,'Value',1);
end
guidata(handles.warp_GUI,handles);
    
% --------------------------------------------------------------------

function [Results] = WarpData(RawData,Constraints,mode);
%   function [Results] = WarpData(RawData,Constraints,mode)
%   
%   purpose:
%   Actually performs the warping.
%
%   Input:
%   RawData     -   struct
%   mode        -   'DTW'
%                   'PTW'
%
%   Output:
%   Results     -   struct
%
    if (  nargin < 1 )
        error('call is: [Results] = WarpData(RawData)')
    end
    
    switch( upper(mode) )
    case {'PTW'}
        [Results.WarpedSample,Results.WarpedRef,Results.WarpedPath] = parametricWarp(RawData.Sample(:,1),RawData.Ref(:,1));
        Results.GlobalConstraints = [];
    case {'DTW'}        
       [Results.WarpedSample,Results.WarpedRef,Results.WarpedPath,Results.GlobalConstraints] = ...
                dwt(RawData.Sample,RawData.Ref,Constraints,[],1);
    otherwise
        errordlg('unknown mode')
    end
% --------------------------------------------------------------------    
function content = GetVarFromBase(name);
%   function content = GetVarFromBase(name);
%
%   Purpose:
%   Transfer the contents of a MATLAB base 
%   space variable 'name' to the output of routine.
%   This requires some advanced fumbling 
%   with eval commands.
%
%   Input:
%   name    -   Char string
%               Name of variable in the MATLAB base space.
%
%   Ouput:
%   content -   <any variable type>
%               Contents of the base space variable.
%   
    evalin('base','assignin(''caller'',''varnames'',who)');
    if ( ~isempty(strmatch(name,varnames)) )
        %   if variables exists in the MATLAB base workspace get its contents.
        eval(['evalin(''base'',''assignin(''''caller'''',''''content'''',',name,')'')']);
    end       
% --------------------------------------------------------------------
function CloseWarp_GUI(handles);
    %   Reset root default colors.
    GlobSet = handles.GlobSet
    set(0,'DefaultFigureColor',GlobSet.PrevFigureColor);
    set(0,'defaultUicontrolBackgroundColor',GlobSet.PrevUicontrolBackgroundColor);


% --------------------------------------------------------------------
function varargout = BUTHelp_Callback(handles)
display_file('Help instructions for the DTW GUI',{fullfile(handles.GlobSet.HelpDir,'helptext.htm')})