function [GridMat,AreaIndex,GlobalConstraints,DirPenaltyVal] = InitGrid(Bref,Bsam,GlobalConstraint,fdist)
%   [GridMat,GlobalConstraints,DirPenaltyVal] = InitGrid(Bref,Bsam,GlobalConstraint,fdist)
%
%   purpose:
%   Initialize coordinate grid and return estimate of direction penalty.
%
%   Input:
%   Bsam                -   (m x k) vector
%                           New sample.
%
%   Bref                -   (n x k) vector
%                           Reference sample.
%                           n maybe larger of smaller than m
%
%   GlobalConstraint    -   scalar
%                           0 = Do NOT use global constraint.
%                           1 = Use global constraint.
%
%   fdist               -   (optional)
%                           Function handle.
%                           Distance function in grid.
%                           Default: distSS ( = sum of squares )
%
%
%   Output:
%   GridMat             -  (n x m ) matrix
%                          filled coordinate grid.
%
%   GlobalConstraints   -  (2 x 1) vector
%                          GlobalConstraints(1)
%                          Starting point of constraint along first (reference)
%                          axis of coordinate grid.
%                          GlobalConstraints(2)
%                          Starting point of constraint along second (sample)
%                          axis of coordinate grid.
%
%   DirPenaltyVal       -  scalar
%                          Estimate of direction penalty. This is equal to 
%                          average of the value at the filled grid points.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
if ( nargin < 3 )
    error('Call is: [GridMat,GlobalConstraints,DirPenaltyVal] = InitGrid(Bref,Bsam,GlobalConstraint,fdist)')
end
if ( nargin < 4 )
    fdist = @distSS;
end
if ( isempty(fdist) ) 
    fdist = @distSS;
end

% Define n and m of grid.
n      = size(Bref,1);
m      = size(Bsam,1);

if ( GlobalConstraint )

    %   Take global constraints into account.
    GridMat     = ones(n,m)*NaN;
    [ReferenceConstraint,SampleConstraint]=Constraints(n,m);
    GridMat     = triu(GridMat,SampleConstraint)+tril(GridMat,-ReferenceConstraint);
    
else

    %   Use whole grid: no global constraints.
    GridMat             = zeros(n,m);
    ReferenceConstraint = n;
    SampleConstraint    = m;
    
end
GlobalConstraints = [ReferenceConstraint SampleConstraint];

%   This index contains all the gridpoints within the allowed search area.
[I,J]       = find(GridMat==0);
AreaIndex   = [I J];

%   The grid is initialized by filling in the local
%   distances between sample and reference at the gridpoints 
SumGridPoints=0;
for i = 1:length(AreaIndex)
     GridMat(AreaIndex(i,1),AreaIndex(i,2)) = feval(fdist,Bref(AreaIndex(i,1),:),Bsam(AreaIndex(i,2),:));
     SumGridPoints = SumGridPoints + GridMat(AreaIndex(i,1),AreaIndex(i,2));
end
DirPenaltyVal = SumGridPoints / length(AreaIndex);