function [data,seed] = whnoise(npts,sigma,seed)
% [data,seed] = whnoise(npts,sigma,seed)
%
%   Purpose:
%   Generates a column vector with Gaussian distributed,
%   white noise.
%
%   Parameters 
%   npts        - Length of column vector.
%   sigma       - Standard deviation of the noise.
%   seed        - (optional)
%             Seed to start random generator with. In case
%             no seed is passed each time the routine is
%             called another noise realisation will be
%             generated.
%             Default: fix(rand * 2^31)
%
%   Returned parameters:
%   data        - Column vector with noise.
%   seed        - Seed that was used for noise generation.
%
%   Example of use:
%
%   % Plots a white noise sequence (100 datapoints, standard deviation= 0.1)
%   plot(whnoise(100,0.1))
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================

% -------------- Argument checking -----------------------------------
if ( nargin > 3 | nargin < 2 )
    error('Call is: [data,seed] = whnoise(npts,sigma,seed)');
end;

if ( npts <= 0 )
    error('npts must be > 0');
end
if ( sigma < 0 )
    error('sigma must be >= 0');
end
if ( sigma == 0 )
    data = zeros(npts,1);
    return
end

if ( nargin == 2 )
    seed = fix(rand * 2^31);
end;
if ( isempty(seed) )
    seed = fix(rand * 2^31);
end;
% -------------- End of Argument checking -----------------------------------

% Seed the random generator 
randn('seed',seed);

% Generate the noise record
data = sigma * randn(npts,1);
return;
