function [Wsam,Wref,WarpPath,GlobalConstraints,ActDirPenaltyVal] = dwt(Bsam,Bref,options,fdist,ShowWait)
%   function [Wsam,Wref,WarpPath,GlobConstraints,ActDirPenaltyVal] = dwt(Bsam,Bref,options,fdist,ShowWait)
%
%   Input:
%   Bsam        -   (m x 1) vector
%                   New sample.
%
%   Bref        -   (n x 1) vector
%                   Reference sample.
%                   n maybe larger of smaller than m
%
%   options     -   (optional)
%                   Option that determine exact way the warping is
%                   performed:
%                   options(1)  --   0 : Do NOT use global constraints.
%                                    1 : Use global constraints.
%                                    Setting the global contraints on will 
%                                    limit the area of the sample-reference
%                                    coordinate space when searching for the
%                                    best warping path through that space.
%
%                   options(2)  --   0 : Do NOT use normalization.
%                                    1 : Use normalization
%                                    Normalization 
%                                   
%
%                   options(3)  --   0 : Do NOT use direction penalty.
%                                    1 : Use direction penalty
%                   options(4)  --   Value of direction penalty, only used
%                                    when options(3) is 1. When this value
%                                    is set to NaN, the routine will use
%                                    default value.
%                   Defaults: [0,0,0,0]
%
%   
%   fdist       -   (optional)
%                   Function handle.
%                   Distance function in grid.
%                   Default: distSS ( = sum of squares )
%
%   ShowWait    -   (optional)
%                   0   -   Do NOT show wait bar.
%                   1   -   Show wait bar.
%
%   Output:
%   WarpPath    -   (k x 2) matrix
%                   XY-coordinates of the optimal path through the
%                   (n x m) space to warp sample vector on the
%                   reference vector.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
%
%   ---- Argument checking -----
if ( nargin < 2 )
    error('Call is: [Wsam,Wref,WarpPath,GlobConstr,DirPenalty] = dwt(Bsam,Bref,options,fdist)')
end
if ( nargin < 3 )
    % Set options ---> No global constraint, No direction penalty, No direction penalties
    options = [0,0,0,NaN];    
end
if ( nargin < 4 )
    fdist = @distSS;
end
if ( isempty(fdist) ) 
    fdist = @distSS;
end
if ( nargin < 5 )
    ShowWait = 0;
end
if ( min(size(Bref)) == 1 ) 
    Bref = Bref(:);
end    
if ( min(size(Bsam)) == 1 ) 
    Bsam = Bsam(:);
end    
nvarBref = size(Bref,2);
if ( nvarBref > 1 ) 
    rBref = range(Bref);
    mBref = min(Bref);
    for ( k = 1:nvarBref )
        Bref(:,k) = (Bref(:,k) - mBref(k)) / rBref(k);
    end
    rBsam = range(Bsam);
    mBsam = min(Bsam);
    for ( k = 1:nvarBref )
        Bsam(:,k) = (Bsam(:,k) - mBsam(k)) / rBsam(k);
    end
end

% Define n and m of grid.
n      = size(Bref,1);
m      = size(Bsam,1);

%   Get options
options = [options(:);zeros(4,1)];
options = options(1:4);

GlobalConstraint    = options(1);
Normalization       = options(2);
DirPenalty          = options(3);
DirPenaltyValue     = options(4);
GuessDirPenalty     = isnan(DirPenaltyValue);

%   ---- Start of code -----

%   These are the constrains and their weights. 
%   mask(1) = vertical step
%   mask(2) = diagonal step
%   mask(3) = horizontal step
mask    = [-1 0;-1 -1;0 -1];
if ( Normalization )
    weight  = [1,2,1];
else
    weight  = [1,1,1];
end
nmask   = size(mask,1);

[GridMat,AreaIndex,GlobalConstraints,ActDirPenaltyVal] = InitGrid(Bref,Bsam,GlobalConstraint,fdist);

%   AreaIndex is a 2 column matrix. Each row of the matrix
%   defines a gridpoint in the coordinate space that maybe 
%   used for finding the optimal path.

if ( DirPenalty )
    if ( GuessDirPenalty )
        dp      = [ActDirPenaltyVal,ActDirPenaltyVal];
    else
        dp      = [DirPenaltyValue,0,DirPenaltyValue];
    end        
else
    dp      = [0,0,0];
end

%   The first gridpoint is given this value
GridMat(1,1) = 2 * GridMat(1,1)/2;

% The first row and the first column are filled using dynamic programming.
%   ... first row.
for ( im = 2:GlobalConstraints(2) ) 
    ndxprev         = [1,im] + mask(3,:);
    GridMat(1,im)   = GridMat(ndxprev(1),ndxprev(2)) + GridMat(1,im) + dp(3);                            
end
%   ... first column.
for ( in = 2:GlobalConstraints(1) ) 
    ndxprev         = [in,1] + mask(1,:);
    GridMat(in,1)   = GridMat(ndxprev(1),ndxprev(2)) + GridMat(in,1) + dp(1);                            
end

% The first row and first column are removed for the search area, because they
% have already been calculated
[i,j]           = find(AreaIndex(:,1)==1);
AreaIndex(i,:)  = [];
[i,j]           = find(AreaIndex(:,2)==1);
AreaIndex(i,:)  = [];

%   The remaining gridpoints are used and the cost function is
%   applied using the local constrains as described above.
ShowWait = 1;
if ( ShowWait )
    hwait = waitbar(0,'Please wait...');
    idel  = floor(length(AreaIndex)/20);
end    
for i = 1:length(AreaIndex)
    if ( rem(i,idel) == 0 & ShowWait)
        waitbar(i/length(AreaIndex),hwait)
    end        
    in = AreaIndex(i,1);
    im = AreaIndex(i,2);
    for ( imask = 1:nmask )
        ndxprev     = [in,im] + mask(imask,:);
        D(imask)    = GridMat(ndxprev(1),ndxprev(2)) + GridMat(in,im) * weight(imask) + dp(imask);                            
    end
    [Temp,imin]     = min(D);
    GridMat(in,im)  = D(imin);
end
if ( ShowWait )
    close (hwait)
end    


% Now that the grid matrix is filled, work back through it to 
% extract the optimal warping path. The optimal path is stored
% in a two column matrix ("warppath"). 
% The first column of "warppath" represents the coordinate 
% along the reference vector (running between 1 and n) and
% the second column of "warppath" represents the
% coordinate along the sample vector.
ipath           = 2;
WarpPath        = [];
OptimalValues   = GridMat(n,m);
ndxCurrentPoint = [n m];
while (  any(ndxCurrentPoint ~= [1,1]) )

    D        = [];
    WarpPath = [WarpPath;ndxCurrentPoint];
    for ( imask = 1:nmask )
        ndxprev         = ndxCurrentPoint + mask(imask,:);
        CheckArgument   = [all(ndxprev==0) xor(ndxprev(1),ndxprev(2)) isnan(ndxprev)];
        if ( all(CheckArgument==0 ) ) 
            D(imask) = GridMat(ndxprev(1),ndxprev(2));           
        else
            D(imask) = Inf;           
        end
    end 
    [OptimalValues(ipath),imin] = min(D);
    ndxCurrentPoint             = ndxCurrentPoint + mask(imin,:);
    ipath                       = ipath + 1;        
end
WarpPath    = [WarpPath;ndxCurrentPoint];
OptimalPath = (fliplr(WarpPath'))';
Wref        = Bref(OptimalPath(:,1),:);
Wsam        = Bsam(OptimalPath(:,2),:);
if ( nvarBref > 1 ) 
    for ( k = 1:nvarBref )
        Wref(:,k) = rBref(k) * Wref(:,k) + mBref(k);    
        Wsam(:,k) = rBsam(k) * Wsam(:,k) + mBsam(k);    
    end    
end    
return