function [newextrema] = paxis(extrema,Xexc,Yexc)
%   [newextrema] = paxis(extrema,Xexc,Yexc)
%
%   Purpose:
%   Improved scaling of 2-D MATLAB plots: P(retty)AXIS.
%
%   Input:
%   extrema   -   (optional)
%                 (4 x 1) Array 
%                 lower limit x-axis
%                 upper limit x-axis
%                 lower limit y-axis
%                 upper limit y-axis
%                 Default: values returned by command axis('tight')
%
%   Xexc      -   (optional)
%                 scalar
%                 Percentage used for enlarging the true x-range.
%                 Default: the value global variable Xexcess
%                          in case this global variable does not exist
%                          value is set to 3
%
%   Yexc      -   (optional) 
%                 scalar
%                 Percentage used for enlarging the true y-range.
%                 Default: the global variable Yexcess
%                          in case this global variable does not exist
%                          value is set to 3
%
%   Global variables:
%   Xexcess   - Increase of x plot range in percent.        
%   Yexcess   - Increase of y plot range in percent.
%
%   Description:
%   Automatic scaling of 2-D MATLAB plots often yields plots in which
%   the x and y ranges are much too large. Which this routine it is
%   possible to set the size of an axis with respect to the minimum and
%    maximum values plotted along that axis.
% 
%   Examples of use:
%   ---  Example 1 ---
%       lims = paxis(axis,10,5)
%       axis(lims)
%
%       Here 'axis' is the standard MATLAB command. The new x-axis range
%       is 10% larger than the previous x-axis. The y-axis range is
%       5% larger.
%
%   --- Example 2 ---
%       global Xexcess Yexcess
%       Xexcess = 10
%       Yexcess = 5
%       plot(x,y)
%       lims = paxis(axis)
%       axis(lims)
%
%       Same result as for example 1 only the percentages are passed as 
%       by the global variables. Advantage: all plots for which paxis()
%       is called will be scaled in the same way.
%
%   --- Example 3---
%       axis(paxis)
%       Again 'axis' is the standard MATLAB command. The new x-axis range
%       is 3% (= default) larger than range of the current x-data 
%       plotted in the figure. Idem for y-axis.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================

%
global Xexcess Yexcess
if ( nargin < 1 )
    extrema  = GetExtr;
end
if ( isempty(extrema) )
    extrema  = GetExtr;
end
extrema = extrema(:);

Xdef = 0;
if ( exist('Xexc') == 1 )
    if ( ~isempty(Xexc) )
        Xdef = 3;
    end
end 
if ( Xdef == 0 )
    if ( ~isempty(Xexcess) )
        Xexc = Xexcess;     
    else
        Xexc = 3;       
    end
end
Ydef = 0;
if ( exist('Yexc') == 1 )
    if ( ~isempty(Yexc) )
        Ydef = 3;
    end
end 
if ( Ydef == 0 )
    if ( ~isempty(Yexcess ) )
        Yexc = Yexcess;     
    else
        Yexc = 3;       
    end
end
xmin    = extrema(1);
xmax    = extrema(2);
ymin    = extrema(3);
ymax    = extrema(4);
Xcorr   = Xexc * 0.01 * abs(xmax-xmin);
Ycorr   = Yexc * 0.01 * abs(ymax-ymin);
xmin    = xmin - Xcorr;
xmax    = xmax + Xcorr;
ymin    = ymin - Ycorr;
ymax    = ymax + Ycorr;
if ( all( ~isnan([xmin,xmax,ymin,ymax])) )
   axis([xmin,xmax,ymin,ymax]);
end    
if ( nargout == 1 )
    newextrema = [xmin,xmax,ymin,ymax];
end

function limits = GetExtr
% Get the upper and lower limits of the data draw in current set of axes
ax          = gca;
ch          = get(ax,'children');
limits      = [inf -inf inf -inf inf -inf];
hasdepth    = 0;
for i=1:length(ch),
    switch get(ch(i),'type')
    
    case {'patch','surface','line'}
        data{i} = get(ch(i),{'xdata','ydata','zdata'});
        
    case 'image'
        data{i} = get(ch(i),{'xdata','ydata','cdata'});

        % Offset data limits by half a pixel
        siz = size(data{i}{3});
        data{i}{1} = [min(data{i}{1}) max(data{i}{1})];
        data{i}{2} = [min(data{i}{2}) max(data{i}{2})];
        dx = diff(data{i}{1}); dy = diff(data{i}{2});
        data{i}{1} = data{i}{1} + [-dx dx]/(siz(2)-1)/2;
        data{i}{2} = data{i}{2} + [-dy dy]/(siz(1)-1)/2;
        data{i}{3} = [];
        
    case 'text'
        % Ignore text like the axes object does.  This
        % handles filtering out the xlabel, ylabel, zlabel, and
        % title objects as well.
        data{i} = cell(1,3);
        
    otherwise
        data{i} = cell(1,3);
        
    end
    if ~isempty(data{i}{1})
        limits(1:2) =   [min(limits(1),min(data{i}{1}(:))) ...
                         max(limits(2),max(data{i}{1}(:)))];
    end
    if ~isempty(data{i}{2})
        limits(3:4) =   [min(limits(3),min(data{i}{2}(:))) ...
                         max(limits(4),max(data{i}{2}(:)))];
    end
    if ~isempty(data{i}{3}),
        limits(5:6) = [min(limits(5),min(data{i}{3}(:))) ...
                       max(limits(6),max(data{i}{3}(:)))];
        if limits(5)~=limits(6), hasdepth = 1; end
    end
end

% Protection same values for axis limit. 
ndx = find(diff(limits)==0 & [1 0 1 0 1]);
if ~isempty(ndx)
  limits(ndx+1) = limits(ndx)+1;
  limits(ndx) = limits(ndx)-1;
end
if all(isfinite(limits(1:4))),
  set(ax,'xlim',limits(1:2),'ylim',limits(3:4))
end
if (hasdepth)
    set(ax,'zlim',limits(5:6))
end